package com.aidex.common.gps.common;

import com.aidex.common.constant.Constants;
import com.aidex.common.core.redis.RedisCache;
import com.aidex.framework.cache.ConfigUtils;
import com.google.common.base.Joiner;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

@Component
public class GeneratorGspApi {

    @Autowired
    private RedisCache redisCache;
    private final String apiHost = ConfigUtils.getConfigByKey("sys.gps.api.url").getConfigValue();

    // 获取当前坐标
    private final String server = "/GetDataService.aspx";
    private final String mapServer = "/RouterPass.ashx";

    public String GenerationMapUrl(String method,String macId,String mapType){
        String mds = redisCache.getStringValue(Constants.GPS_TOKEN_PREFIX);
        Map<String, String> parmas = new HashMap<>();
        parmas.put("method",method);
        parmas.put("mds",mds);
        parmas.put("macid",macId);
        parmas.put("url","http://www.18gps.net/");
        parmas.put("mapType",mapType);

        StringBuilder builder = new StringBuilder(apiHost);
        builder.append(mapServer);
        builder.append("?");
        builder.append(GeneratorParams(parmas));
        return builder.toString();
    }


    public Map<String, String> GenerationParams(String method,String api){
        String mds = redisCache.getStringValue(Constants.GPS_TOKEN_PREFIX);
        Map<String, String> parmas = new HashMap<>();
        parmas.put("method",method);
        parmas.put("w",api);
        parmas.put("mds",mds);
        return parmas;
    }

    public String GeneratorApi(){
        StringBuilder builder = new StringBuilder(apiHost);
        builder.append(server);
        return builder.toString();
    }

    public Map<String, String> GeneratorQueryParams(String api){
        return GenerationParams(RquestMethod.query,api);
    }

    public Map<String, String> GeneratorSignParams(String api){
        return GenerationParams(RquestMethod.sign,api);
    }

    public interface RquestMethod {
        String query = "QueryApi";
        String sign = "SignApi";
        String follow = "Follow";
        String playback = "Playback";
    }

    public interface GpsMapType {
        String GOOGLECN = "GOOGLECN";
        String BAIDU = "BAIDU";
        String GAODE = "GAODE";
        String QQ = "QQ";
        String DEFAULT = "";

    }


    public static String GeneratorParams(Map<String, String> params){
        // TODO 如果要编码的话自己加下编码逻辑
        return Joiner.on("&")
                // 用指定符号代替空值,key 或者value 为null都会被替换
                .useForNull("")
                .withKeyValueSeparator("=")
                .join(params);
    }

}
